#pragma once

//CAN speed - register values for microprocessor inside the device
//                                   (50%),         (62.5%),       (75%),         (87.5%)} 
const BYTE cCanSpeed[][4][4] = {{{49, 0, 5, 7}, {49, 0, 7, 5}, {49 ,2, 7, 3}, {49, 4, 7, 1}},  //10Baud 0 (BRP, PRS, PHS1, PHS2)
                                {{24, 0, 5, 7}, {24, 0, 7, 5}, {24, 2, 7, 3}, {24, 4, 7, 1}},  //20Baud 1
                                {{14, 0, 5, 7}, {14, 0, 7, 5}, {14, 2, 7, 3}, {14, 4, 7, 1}},  //33Baud 2
                                {{ 9, 0, 5, 7}, { 9, 0, 7, 5}, { 9, 2, 7, 3}, { 9, 4, 7, 1}},  //50Baud 3
                                {{ 5, 0, 5, 7}, { 5, 0, 7, 5}, { 9, 2, 7, 3}, { 5, 4, 7, 1}},  //83Baud 4
                                {{ 4, 0, 5, 7}, { 4, 0, 7, 5}, { 5, 2, 7, 3}, { 4, 4, 7, 1}},  //100Baud 5
                                {{ 3, 0, 5, 7}, { 3, 0, 7, 5}, { 3, 2, 7, 3}, { 3, 4, 7, 1}},  //125Baud 6
                                {{ 1, 0, 5, 7}, { 1, 0, 7, 5}, { 1, 2, 7, 3}, { 1, 4, 7, 1}},  //250Baud 7
                                {{ 0, 0, 5, 7}, { 0, 0, 7, 5}, { 0, 2, 7, 3}, { 0, 4, 7, 1}},  //500Baud 8
                                {{ 0, 0, 2, 4}, { 0, 0, 3, 3}, { 0, 0, 4, 2}, { 0, 4, 1, 1}},  //800Baud 9
                                {{ 0, 0, 1, 3}, { 0, 0, 2, 2}, { 0, 0, 3, 1}, { 0, 3, 1, 0}}}; //1000Baud 10

//Message format
//   0       1      2      3          4    5
// START1,START2,CONTROL,LENGTH,DATA,END1,END2

const BYTE cSTART1 = 0xFA;
const BYTE cSTART2 = 0xF5;
const BYTE cEND1 = 0xAF;
const BYTE cEND2 = 0x5F;

//Bytes position
const BYTE cPosSTART1 = 0x00;
const BYTE cPosSTART2 = 0x01;
const BYTE cPosCONTROL = 0x02;
const BYTE cPosLENGTH = 0x03;
const BYTE cPosData = 0x04;
const BYTE cPosCRC = 0x04; //Real possition is increased by data length
const BYTE cPosEND1 = 0x05; //Real possition is increased by data length
const BYTE cPosEND2 = 0x06; //Real possition is increased by data length

//Whole message length
const BYTE cMsgLENGTH = 0x07; //Number of bytes in message except data array

//Commands transmitted to device
const BYTE cReady = 0x00; //Test if device is ready
const BYTE cInitCan = 0x01; //Initialization CAN driver
const BYTE cInitMC0_7 = 0x02; //Initialization MC0-MC7
const BYTE cInitMC8_14 = 0x03; //Initialization MC8-MC14
const BYTE cInitTime = 0x04; //Initialization Time
const BYTE cDisCan = 0x06; //Disable CAN driver
const BYTE cDisMC0_7 = 0x07; //Disable M0 - MC7 (Receive)
const BYTE cDisMC8_14 = 0x08; //Disable M8 - MC14 (Tranceive)
const BYTE cPause = 0x0A; //Pause receive and transceive

//Commands received from device
const BYTE cCAN_RX = 0x0B; //Receive Can Data
const BYTE cCAN_TX = 0x0C; //Transceive Can Data
const BYTE cCAN_ERROR = 0x0D; //Can Error
