#include "DlgMain.h"
#include "Consts.h"
#include "Conversions.h"


//Global thread-functions prototypes
DWORD WINAPI ThreadTimerRutine(LPVOID lpParameter);
DWORD WINAPI ThreadReceiveRutine(LPVOID lpParameter);


/**************************************************************************************************
    Description: Constructor - set default value for all member pointers
---------------------------------------------------------------------------------------------------
    Inputs: hInstance - Main window handler
    Output: -
**************************************************************************************************/
CDlgMain::CDlgMain(HINSTANCE hInstance)
{
    m_hInstance = hInstance;
    m_FT_Handle = NULL;
    m_hThreadTimer = NULL;
    m_hThreadReceive = NULL;
}

/**************************************************************************************************
    Description: Destructor - release allocated memory by all member pointers
---------------------------------------------------------------------------------------------------
    Inputs: -
    Output: -
**************************************************************************************************/
CDlgMain::~CDlgMain()
{
    if (m_hThreadTimer != NULL)
    {
        CloseHandle(m_hThreadTimer);
    }
    if (m_hThreadReceive != NULL)
    {
        CloseHandle(m_hThreadReceive);
    }
}

/**************************************************************************************************
    Description: Attach class members to window controls and set default states
---------------------------------------------------------------------------------------------------
    Inputs: Standard arguments for message handler
    Output: Allways 0 (zero)
**************************************************************************************************/
LRESULT CDlgMain::OnInitDialog(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
{
    //Set icons
    HICON hIcon = (HICON)LoadImage(m_hInstance, MAKEINTRESOURCE(IDI_MAIN_ICON), IMAGE_ICON, 16, 16, LR_DEFAULTCOLOR);
    SetIcon(hIcon, FALSE); //Small icon
    hIcon = (HICON)LoadImage(m_hInstance, MAKEINTRESOURCE(IDI_MAIN_ICON), IMAGE_ICON, 32, 32, LR_DEFAULTCOLOR);
    SetIcon(hIcon, TRUE); //Large icon

    //Create timer to peridically checking if device is available
    m_bDevicePresent = FALSE;
    m_dwThreadTimerID = 0;
    m_bThreadTimerPaused = FALSE;
    m_bThreadTimerTerminate = FALSE;
    m_hThreadTimer = CreateThread(NULL, 0, ThreadTimerRutine, this, 0, &m_dwThreadTimerID);
    m_bCanPause = TRUE;

    //Editboxes
    m_ControlEdit_ArrivalTime.Attach(GetDlgItem(IDC_EDIT_ARRIVAL_TIME));
    m_ControlEdit_IdReceive.Attach(GetDlgItem(IDC_EDIT_ID_RECEIVE));
    m_ControlEdit_DataReceive.Attach(GetDlgItem(IDC_EDIT_DATA_RECEIVE));
    m_ControlEdit_Period.Attach(GetDlgItem(IDC_EDIT_PERIOD));
    m_ControlEdit_IdTransmit.Attach(GetDlgItem(IDC_EDIT_ID_TRANSMIT));
    m_ControlEdit_DataTransmit.Attach(GetDlgItem(IDC_EDIT_DATA_TRANSMIT));
    m_ControlEdit_Tag.Attach(GetDlgItem(IDC_EDIT_TAG));
    m_ControlEdit_Mask.Attach(GetDlgItem(IDC_EDIT_MASK));
    //Buttons
    m_ControlButton_Transmit.Attach(GetDlgItem(IDC_BUTTON_TRANSMIT));
    m_ControlButton_Abort.Attach(GetDlgItem(IDC_BUTTON_ABORT));
    m_ControlButton_Init.Attach(GetDlgItem(IDC_BUTTON_INIT));
    m_ControlButton_Pause.Attach(GetDlgItem(IDC_BUTTON_PAUSE));
    m_ControlButton_Stop.Attach(GetDlgItem(IDC_BUTTON_STOP));
    m_ControlButton_Clear.Attach(GetDlgItem(IDC_BUTTON_CLEAR));
    //Radiobuttons
    m_ControlRadio_Type20A.Attach(GetDlgItem(IDC_RADIO_TYPE_20A));
    m_ControlRadio_Type20B.Attach(GetDlgItem(IDC_RADIO_TYPE_20B));
    m_ControlRadio_High.Attach(GetDlgItem(IDC_RADIO_HIGH));
    m_ControlRadio_Low.Attach(GetDlgItem(IDC_RADIO_LOW));
    m_ControlRadio_OneWire.Attach(GetDlgItem(IDC_RADIO_ONE_WIRE));
    m_eOneWireType = ONE_WIRE_TYPE_NONE;
    //Checkboxes
    m_ControlCheck_Periodic.Attach(GetDlgItem(IDC_CHECK_PERIODIC));
    m_ControlCheck_Remote.Attach(GetDlgItem(IDC_CHECK_REMOTE));
    m_ControlCheck_ShowTransmited.Attach(GetDlgItem(IDC_CHECK_SHOW_TRANSMITED));
    m_ControlCheck_ListeningMode.Attach(GetDlgItem(IDC_CHECK_LISTENING_MODE));
    //Labels
    m_ControlLabel_Errors.Attach(GetDlgItem(IDC_LABEL_MESSAGES_ERRORS));
    m_ControlLabel_Count.Attach(GetDlgItem(IDC_LABEL_MESSAGES_COUNT));
    //Comboboxes
    m_ControlCombo_BitRate.Attach(GetDlgItem(IDC_COMBO_BITRATE));

    //Set default states
    m_ControlButton_Transmit.SetFocus();
    m_ControlRadio_High.SetCheck(BST_CHECKED);
    m_ControlRadio_Type20A.SetCheck(BST_CHECKED);
    m_ControlEdit_ArrivalTime.SetWindowText(TEXT("--:--:--.---"));
    m_ControlEdit_IdReceive.SetWindowText(TEXT("--------"));
    m_ControlEdit_DataReceive.SetWindowText(TEXT("-- -- -- -- -- -- -- --"));
    m_ControlEdit_Period.SetWindowText(TEXT("100"));
    m_ControlEdit_IdTransmit.SetWindowText(TEXT("1FE"));
    m_ControlEdit_DataTransmit.SetWindowText(TEXT("01 02"));
    m_ControlEdit_Tag.SetWindowText(TEXT("0"));
    m_ControlEdit_Mask.SetWindowText(TEXT("0"));
    m_ControlCombo_BitRate.AddString(TEXT("10"));
    m_ControlCombo_BitRate.AddString(TEXT("20"));
    m_ControlCombo_BitRate.AddString(TEXT("33.3"));
    m_ControlCombo_BitRate.AddString(TEXT("50"));
    m_ControlCombo_BitRate.AddString(TEXT("83.3"));
    m_ControlCombo_BitRate.AddString(TEXT("100"));
    m_ControlCombo_BitRate.AddString(TEXT("125"));
    m_ControlCombo_BitRate.AddString(TEXT("250"));
    m_ControlCombo_BitRate.AddString(TEXT("500"));
    m_ControlCombo_BitRate.AddString(TEXT("800"));
    m_ControlCombo_BitRate.AddString(TEXT("1000"));

    //TODO: Set controls hints (not easy in ATL)

    bHandled = TRUE;
    return 0;
}

/**************************************************************************************************
    Description: Load all settings from ini file
---------------------------------------------------------------------------------------------------
    Inputs: Standard arguments for message handler
    Output: Allways 0 (zero)
**************************************************************************************************/
LRESULT CDlgMain::OnShowWindow(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
{
    //Restore window position
    int x = GetPrivateProfileInt(TEXT("Global"), TEXT("Left"), 0, ResStr(IDS_CONFIG_FILE_NAME));
    int y = GetPrivateProfileInt(TEXT("Global"), TEXT("Top"), 0, ResStr(IDS_CONFIG_FILE_NAME));
    SetWindowPos(HWND_NOTOPMOST, x, y, 0, 0, SWP_NOZORDER | SWP_NOSIZE);

    //Restore window state
    INT nStyle = GetPrivateProfileInt(TEXT("Global"), TEXT("WindowStyle"), 0, ResStr(IDS_CONFIG_FILE_NAME));
    switch (nStyle)
    {
    case WS_MINIMIZE:
        SendMessage(WM_SYSCOMMAND, SC_MINIMIZE, 0);
        break;
    //case WS_MAXIMIZE:
    //    SendMessage(WM_SYSCOMMAND, SC_MAXIMIZE, 0);
    //    break;
    }

    //Restore controls setting
    INT nTmp;
    CString strTmp;

    nTmp = GetPrivateProfileInt(TEXT("CAN"), TEXT("Speed"), 5, ResStr(IDS_CONFIG_FILE_NAME));
    m_ControlCombo_BitRate.SetCurSel(nTmp);

    nTmp = GetPrivateProfileInt(TEXT("CAN"), TEXT("Listening"), 0, ResStr(IDS_CONFIG_FILE_NAME));
    m_ControlCheck_ListeningMode.SetCheck((nTmp == 0) ? BST_UNCHECKED : BST_CHECKED);

    GetPrivateProfileString(TEXT("CAN"), TEXT("Tag"), TEXT("0"), strTmp.GetBuffer(MAX_TAG_LENGTH), MAX_TAG_LENGTH, ResStr(IDS_CONFIG_FILE_NAME));
    m_ControlEdit_Tag.SetWindowText(strTmp);

    GetPrivateProfileString(TEXT("CAN"), TEXT("Mask"), TEXT("0"), strTmp.GetBuffer(MAX_MASK_LENGTH), MAX_MASK_LENGTH, ResStr(IDS_CONFIG_FILE_NAME));
    m_ControlEdit_Mask.SetWindowText(strTmp);

    nTmp = GetPrivateProfileInt(TEXT("CAN"), TEXT("2.0A"), 1, ResStr(IDS_CONFIG_FILE_NAME));
    m_ControlRadio_Type20A.SetCheck((nTmp == 0) ? BST_UNCHECKED : BST_CHECKED);
    m_ControlRadio_Type20B.SetCheck((m_ControlRadio_Type20A.GetCheck() == BST_CHECKED) ? BST_UNCHECKED : BST_CHECKED);

    //Create timer to peridically checking if device is available
    m_dwRxDataBufferPos = 0;
    m_dwThreadReceiveID = 1;
    m_bThreadReceivePaused = FALSE;
    m_bThreadReceiveTerminate = FALSE;
    m_hThreadReceive = CreateThread(NULL, 0, ThreadReceiveRutine, this, 0, &m_dwThreadReceiveID);

    return 0;
}

/**************************************************************************************************
    Description: Save application settings and stop all threads
---------------------------------------------------------------------------------------------------
    Inputs: Standard arguments for message handler
    Output: Allways 0 (zero)
**************************************************************************************************/
LRESULT CDlgMain::OnClose(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
{
    CString strTmp;

    //Store window position
    RECT Rect;
    GetWindowRect(&Rect);
    strTmp.Format(TEXT("%d"), Rect.left);
    WritePrivateProfileString(TEXT("Global"), TEXT("Left"), strTmp, ResStr(IDS_CONFIG_FILE_NAME));
    strTmp.Format(TEXT("%d"), Rect.top);
    WritePrivateProfileString(TEXT("Global"), TEXT("Top"), strTmp, ResStr(IDS_CONFIG_FILE_NAME));

    //Store window state
    LONG lLong = GetWindowLong(GWL_STYLE);
    if ((lLong & WS_MINIMIZE) != 0)
    {
        strTmp.Format(TEXT("%d"), WS_MINIMIZE);
    }
    else
    {
        //if ((lLong & WS_MAXIMIZE) != 0)
        //{
        //    strTmp.Format(TEXT("%d"), WS_MAXIMIZE);
        //}
        //else
        {
            strTmp = TEXT("0");
        }
    }
    WritePrivateProfileString(TEXT("Global"), TEXT("WindowStyle"), strTmp, ResStr(IDS_CONFIG_FILE_NAME));

    //Restore controls setting
    strTmp.Format(TEXT("%d"), m_ControlCombo_BitRate.GetCurSel());
    WritePrivateProfileString(TEXT("CAN"), TEXT("Speed"), strTmp, ResStr(IDS_CONFIG_FILE_NAME));

    strTmp.Format(TEXT("%d"), (m_ControlCheck_ListeningMode.GetCheck() == BST_CHECKED) ? 1 : 0);
    WritePrivateProfileString(TEXT("CAN"), TEXT("Listening"), strTmp, ResStr(IDS_CONFIG_FILE_NAME));

    m_ControlEdit_Tag.GetWindowText(strTmp.GetBuffer(MAX_TAG_LENGTH), MAX_TAG_LENGTH);
    WritePrivateProfileString(TEXT("CAN"), TEXT("Tag"), strTmp, ResStr(IDS_CONFIG_FILE_NAME));

    m_ControlEdit_Mask.GetWindowText(strTmp.GetBuffer(MAX_MASK_LENGTH), MAX_MASK_LENGTH);
    WritePrivateProfileString(TEXT("CAN"), TEXT("Mask"), strTmp, ResStr(IDS_CONFIG_FILE_NAME));

    strTmp.Format(TEXT("%d"), (m_ControlRadio_Type20A.GetCheck() == BST_CHECKED) ? 1 : 0);
    WritePrivateProfileString(TEXT("CAN"), TEXT("2.0A"), strTmp, ResStr(IDS_CONFIG_FILE_NAME));

    if (m_bDevicePresent)
    {
        m_bThreadTimerTerminate = TRUE;
        Sleep(200); //Wait for thread termination
        OnButtonStopClicked(0, IDC_BUTTON_STOP, m_ControlButton_Stop, bHandled);
        FT_Close(m_FT_Handle);
    }

    EndDialog(0);
    return 0;
}

/**************************************************************************************************
    Description: Send one message or start periodical message transmiting if 'Transmit'
                 button is clicked
---------------------------------------------------------------------------------------------------
    Inputs: Standard arguments for notify handler
    Output: Allways 0 (zero)
**************************************************************************************************/
LRESULT CDlgMain::OnButtonTransmitClicked(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled)
{
    if (m_bDevicePresent)
    {
        CString strPeriod;
        m_ControlEdit_Period.GetWindowText(strPeriod.GetBuffer(MAX_PERIOD_LENGTH), MAX_PERIOD_LENGTH);
        INT nPeriod = StrToInt(strPeriod);

        CString strID;
        m_ControlEdit_IdTransmit.GetWindowText(strID.GetBuffer(MAX_ID_LENGTH), MAX_ID_LENGTH);
        INT nID = HexStrToInt(strID);

        BYTE byCanData[MAX_DATA_LENGTH];
        CString strData;
        m_ControlEdit_DataTransmit.GetWindowText(strData.GetBuffer(MAX_DATA_LENGTH), MAX_DATA_LENGTH);
        INT nCount = HexStrToArray(strData, byCanData, MAX_DATA_LENGTH);

        BYTE byData[MAX_MESSAGE_SIZE - cMsgLENGTH];
        byData[0] = 0; //MCx [0..7]
        byData[1] = nPeriod / 256; // Period
        byData[2] = nPeriod % 256;
        byData[3] = (m_ControlCheck_Periodic.GetCheck() == BST_CHECKED) ? 0x03 : 0x01; //Sys - Periodic frame
        //byData[3] |= (m_ControlCheck_Immediately.GetCheck() == BST_CHECKED) ? 0x04 : 0x00; //Sys - Immediate transmition
        byData[3] |= (m_ControlCheck_ShowTransmited.GetCheck() == BST_CHECKED) ? 0x08 : 0x00; //Sys - Listening
        CodeID(nID, m_ControlCheck_Remote.GetCheck() == BST_CHECKED, &byData[4], &byData[5], &byData[6], &byData[7]);
        byData[8] = nCount + ((m_ControlRadio_Type20A.GetCheck() == BST_CHECKED) ? 0x00 : 0x10); //Length
        for (INT nIndex = 0; nIndex < nCount; nIndex++)
        {
            byData[9 + nIndex] = byCanData[nIndex];
        }

        SendMessageToMC(cInitMC8_14, byData, 0x11/*8 + nCount*/);
        Sleep(50);
    }
    else
    {
        MessageBox(ResStr(IDS_E_DEVICE_NOT_PRESENT), ResStr(IDS_ERROR), MB_OK | MB_ICONERROR);
    }
    return 0;
}

/**************************************************************************************************
    Description: Stop periodical message transmiting if 'Abort' button is clicked
---------------------------------------------------------------------------------------------------
    Inputs: Standard arguments for notify handler
    Output: Allways 0 (zero)
**************************************************************************************************/
LRESULT CDlgMain::OnButtonAbortClicked(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled)
{
    DisableTransmit(0); //We use channel 0 only
    return 0;
}

/**************************************************************************************************
    Description: Initiate connected device if 'Init' button is clicked
---------------------------------------------------------------------------------------------------
    Inputs: Standard arguments for notify handler
    Output: Allways 0 (zero)
**************************************************************************************************/
LRESULT CDlgMain::OnButtonInitClicked(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled)
{
    if (m_bDevicePresent)
    {
        //Clean RX buffer
        m_dwRxDataBufferPos = 0;

        //Test ready
        BYTE byData[MAX_MESSAGE_SIZE - cMsgLENGTH];
        SendMessageToMC(cReady, byData, 0);
        Sleep(50);

        //Send actual time from your PC
        time_t t;
        time(&t);
        struct tm systime;
        localtime_s(&systime, &t);
        byData[0] = systime.tm_hour;
        byData[1] = systime.tm_min;
        byData[2] = systime.tm_sec;
        byData[3] = 0;// / 256;
        byData[4] = 0;// % 256;
        SendMessageToMC(cInitTime, byData, 5);
        Sleep(50);

        //Initialize Global Register Driver
        DisableTransmitToAll();
        Sleep(50);
        INT nPosIndex = 1; //62.5%
        INT nSpeedIndex = m_ControlCombo_BitRate.GetCurSel();
        if (nSpeedIndex < 0)
        {
            nSpeedIndex = 5;
        }
        byData[0] = cCanSpeed[nSpeedIndex][nPosIndex][0] << 1; //CANBT1 - BRP
        byData[1] = cCanSpeed[nSpeedIndex][nPosIndex][1] << 1; //CANBT2 - PRS
        byData[2] = //CANBT3
            0 + //SMP
            (cCanSpeed[nSpeedIndex][nPosIndex][2] << 1) + //PHS1
            (cCanSpeed[nSpeedIndex][nPosIndex][3] << 4); //PHS2
        byData[3] = (m_ControlCheck_ListeningMode.GetCheck() == BST_CHECKED) ? 0x0A : 0x02; //CANGCON
        byData[4] = 0x32; //CANGIE
        byData[5] = GetDriverSpeed();
        SendMessageToMC(cInitCan, byData, 6);
        Sleep(50);

        //Initialize Receive Message Center
        byData[0] = 0; //MCx [0..7]
        CString strTag;
        m_ControlEdit_Tag.GetWindowText(strTag.GetBuffer(MAX_TAG_LENGTH), MAX_TAG_LENGTH);
        CodeID(HexStrToInt(strTag), m_ControlCheck_Remote.GetCheck() == BST_CHECKED, &byData[1], &byData[2], &byData[3], &byData[4]);

        CString strMask;
        m_ControlEdit_Mask.GetWindowText(strMask.GetBuffer(MAX_MASK_LENGTH), MAX_MASK_LENGTH);
        CodeID(HexStrToInt(strMask), m_ControlCheck_Remote.GetCheck() == BST_CHECKED, &byData[5], &byData[6], &byData[7], &byData[8]);
        byData[9] = (m_ControlRadio_Type20A.GetCheck() == BST_CHECKED) ? 0x08 : 0x18; //CANCONCH (2.0A/2.0B + Length)

        SendMessageToMC(cInitMC0_7, byData, 0x0A);
        m_bCanPause = FALSE;
    }
    else
    {
        MessageBox(ResStr(IDS_E_DEVICE_NOT_PRESENT), ResStr(IDS_ERROR), MB_OK | MB_ICONERROR);
    }
    return 0;
}

/**************************************************************************************************
    Description: Toggle between working and paused state if 'Pause' button is clicked
---------------------------------------------------------------------------------------------------
    Inputs: Standard arguments for notify handler
    Output: Allways 0 (zero)
**************************************************************************************************/
LRESULT CDlgMain::OnButtonPauseClicked(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled)
{
    if (m_bDevicePresent)
    {
        //Pause On/Off
        m_bCanPause = ~m_bCanPause;
        BYTE byData[MAX_MESSAGE_SIZE - cMsgLENGTH];
        byData[0] = (m_bCanPause) ? 0xFF : 0x00;
        SendMessageToMC(cPause, byData, 1);
        Sleep(50);
    }
    else
    {
        MessageBox(ResStr(IDS_E_DEVICE_NOT_PRESENT), ResStr(IDS_ERROR), MB_OK | MB_ICONERROR);
    }
    return 0;
}

/**************************************************************************************************
    Description: Stop transmiting and close connection with device if 'Stop' button is clicked
---------------------------------------------------------------------------------------------------
    Inputs: Standard arguments for notify handler
    Output: Allways 0 (zero)
**************************************************************************************************/
LRESULT CDlgMain::OnButtonStopClicked(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled)
{
    if (m_bDevicePresent)
    {
        DisableTransmitToAll();
        Sleep(50);

        //MCx Disable Receive (0) [0..7]
        BYTE byData[MAX_MESSAGE_SIZE - cMsgLENGTH];
        byData[0] = 0;
        SendMessageToMC(cDisMC0_7, byData, 1);
        Sleep(50);

        //Disable Global Register
        SendMessageToMC(cDisCan, byData, 0);
        Sleep(50);

        m_bCanPause = FALSE;

        //Clean RX buffer
        m_dwRxDataBufferPos = 0;
    }
    else
    {
        MessageBox(ResStr(IDS_E_DEVICE_NOT_PRESENT), ResStr(IDS_ERROR), MB_OK | MB_ICONERROR);
    }
    return 0;
}

/**************************************************************************************************
    Description: Clear message and error counters if 'Clear' button is clicked
---------------------------------------------------------------------------------------------------
    Inputs: Standard arguments for notify handler
    Output: Allways 0 (zero)
**************************************************************************************************/
LRESULT CDlgMain::OnButtonClearClicked(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled)
{
    m_ControlLabel_Count.SetWindowText(TEXT("0"));
    m_ControlLabel_Errors.SetWindowText(TEXT("0"));
    return 0;
}

/**************************************************************************************************
    Description: Clear label at the end of 'One Wire' radio button if 'High Speed' radio
                 button is clicked
---------------------------------------------------------------------------------------------------
    Inputs: Standard arguments for notify handler
    Output: Allways 0 (zero)
**************************************************************************************************/
LRESULT CDlgMain::OnRadioHighSpeedClicked(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled)
{
    SetOneWireType(ONE_WIRE_TYPE_NONE);
    return 0;
}

/**************************************************************************************************
    Description: If 'Low Speed' radio button is clicked clear label at the end of 'One Wire'
                 radio button and decrease CAN speed in case if it is higher than 125kbps
---------------------------------------------------------------------------------------------------
    Inputs: Standard arguments for notify handler
    Output: Allways 0 (zero)
**************************************************************************************************/
LRESULT CDlgMain::OnButtonLowSpeedClicked(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled)
{
    if (m_ControlCombo_BitRate.GetCurSel() > 6)
    {
        m_ControlCombo_BitRate.SetCurSel(6);
    }
    SetOneWireType(ONE_WIRE_TYPE_NONE);
    return 0;
}

/**************************************************************************************************
    Description: If 'One Wire' radio button is clicked it set label at the end of 'One Wire'
                 radio button and decrease CAN speed in case if it is higher than 83.3kbps
---------------------------------------------------------------------------------------------------
    Inputs: Standard arguments for notify handler
    Output: Allways 0 (zero)
**************************************************************************************************/
LRESULT CDlgMain::OnButtonOneWireClicked(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled)
{
    if (m_ControlCombo_BitRate.GetCurSel() > 4)
    {
        m_ControlCombo_BitRate.SetCurSel(4);
    }
    m_eOneWireType = (enumOneWireType)((INT)m_eOneWireType + 1);
    if (m_eOneWireType > ONE_WIRE_TYPE_LAST)
    {
        m_eOneWireType = ONE_WIRE_TYPE_FIRST;
    }
    SetOneWireType(m_eOneWireType);
    return 0;
}

/**************************************************************************************************
    Description: Attach some constant to CAN speed in dependence on user setting
---------------------------------------------------------------------------------------------------
    Inputs: -
    Output: speed constant, if fails high speed is choosen
**************************************************************************************************/
INT CDlgMain::GetDriverSpeed()
{
    INT nSpeed = 0x02;
    if (m_ControlRadio_Low.GetCheck() == BST_CHECKED)
    {
        nSpeed = 0x01; //Low speed
    }
    else if (m_ControlRadio_High.GetCheck() == BST_CHECKED)
    {
        nSpeed = 0x02; //High speed
    }
    else if (m_ControlRadio_OneWire.GetCheck() == BST_CHECKED)
    {
        switch (m_eOneWireType) //One wire
        {
        case ONE_WIRE_TYPE_NORMAL:
            nSpeed = 0xC4;
            break;
        case ONE_WIRE_TYPE_HIGH:
            nSpeed = 0x44;
            break;
        case ONE_WIRE_TYPE_WAKE:
            nSpeed = 0x84;
            break;
        }
    }
    return nSpeed;
}

/**************************************************************************************************
    Description: Display one wire type at the end of 'One Wire' radio button label
---------------------------------------------------------------------------------------------------
    Inputs: choosen type constant
    Output: -
**************************************************************************************************/
VOID CDlgMain::SetOneWireType(enumOneWireType eType)
{
    switch (eType)
    {
    case ONE_WIRE_TYPE_NORMAL:
        m_ControlRadio_OneWire.SetWindowText(TEXT("One Wire (Normal)"));
        break;
    case ONE_WIRE_TYPE_HIGH:
        m_ControlRadio_OneWire.SetWindowText(TEXT("One Wire (High)"));
        break;
    case ONE_WIRE_TYPE_WAKE:
        m_ControlRadio_OneWire.SetWindowText(TEXT("One Wire (Wake)"));
        break;
    default:
        m_ControlRadio_OneWire.SetWindowText(TEXT("One Wire"));
        break;
    }
    m_eOneWireType = eType;
}

/**************************************************************************************************
    Description: Pack data into message and send it to message center in connected device
---------------------------------------------------------------------------------------------------
    Inputs: byMsgType - message type (information for message center)
            byData - array of user data
            byDataLength - number of bytes in byData array
    Output: TRUE if success
**************************************************************************************************/
BOOL CDlgMain::SendMessageToMC(BYTE byMsgType, BYTE *pbyData, BYTE byDataLength)
{
    if (!m_bDevicePresent || pbyData == NULL)
    {
        return FALSE;
    }
    m_bThreadTimerPaused = TRUE;

    if (byDataLength > 32)
    {
        byDataLength = 32;
    }

    BYTE byBuffer[cMsgLENGTH + 32];
    byBuffer[cPosSTART1] = cSTART1;
    byBuffer[cPosSTART2] = cSTART2;
    byBuffer[cPosCONTROL] = byMsgType;
    byBuffer[cPosLENGTH] = byDataLength;
    for (BYTE byIndex = 0; byIndex < byDataLength; byIndex++)
    {
        byBuffer[cPosData + byIndex] = pbyData[byIndex];
    }
    byBuffer[byDataLength + cPosCRC] = CRC(byBuffer, byDataLength);
    byBuffer[byDataLength + cPosEND1] = cEND1;
    byBuffer[byDataLength + cPosEND2] = cEND2;

    DWORD dwWrittenBytesCount;
    FT_STATUS ftStatus = FT_Write(m_FT_Handle, &byBuffer, cMsgLENGTH + byDataLength, &dwWrittenBytesCount);

    m_bThreadTimerPaused = FALSE;
    return ((FT_SUCCESS(ftStatus))) ? (dwWrittenBytesCount == cMsgLENGTH + byDataLength) : FALSE;
}

/**************************************************************************************************
    Description: Disable transmitting for one choosen channel
---------------------------------------------------------------------------------------------------
    Inputs: choosem channel
    Output: -
**************************************************************************************************/
VOID CDlgMain::DisableTransmit(BYTE byMCx)
{
    if (m_bDevicePresent)
    {
        BYTE byData[MAX_MESSAGE_SIZE - cMsgLENGTH];
        byData[0] = byMCx;
        SendMessageToMC(cDisMC8_14, byData, 1);
    }
}

/**************************************************************************************************
    Description: Disable transmitting for all channels
---------------------------------------------------------------------------------------------------
    Inputs: -
    Output: -
**************************************************************************************************/
VOID CDlgMain::DisableTransmitToAll()
{
    for (BYTE byIndex = 0; byIndex < 8; byIndex++)
    {
        DisableTransmit(byIndex);
    }
}

/**************************************************************************************************
    Description: Encode ID for device registers
---------------------------------------------------------------------------------------------------
    Inputs: nID - ID for encoding
            bRemote - should be remote frame
            nIDT1..nIDT4 - encoded data
    Output: -
**************************************************************************************************/
VOID CDlgMain::CodeID(INT nID, BOOL bRemote, BYTE *pbyIDT1, BYTE *pbyIDT2, BYTE *pbyIDT3, BYTE *pbyIDT4)
{
    BOOL bType20A = (m_ControlRadio_Type20A.GetCheck() == BST_CHECKED);
    *pbyIDT1 = (bType20A) ? (nID & 0x7F8) >> 3 : (nID & 0x1FE00000) >> 21;
    *pbyIDT2 = (bType20A) ? (nID & 0x007) << 5 : (nID & 0x001FE000) >> 13;
    *pbyIDT3 = (bType20A) ?                  0 : (nID & 0x00001FE0) >>  5;
    *pbyIDT4 = (bType20A) ?                  0 : (nID & 0x0000001F) <<  3;
    *pbyIDT4 = *pbyIDT4 | ((bRemote) ? 4 : 0);
}

/**************************************************************************************************
    Description: Decode ID from device registers
---------------------------------------------------------------------------------------------------
    Inputs: nID - ID for encoding
            bRemote - should be remote frame
            nIDT1..nIDT4 - encoded data
    Output: -
**************************************************************************************************/
DWORD CDlgMain::DecodeID(BYTE byIDT1, BYTE byIDT2, BYTE byIDT3, BYTE byIDT4, BYTE byCONCH, BOOL *pbIDE, BOOL *pbRemote)
{
    DWORD dwID;
    *pbIDE = ((byCONCH & 0x10) == 0x10);
    *pbRemote = ((byIDT4 & 0x04) == 0x04);
    if (*pbIDE)
    {
        dwID = (byIDT1 << 21) | (byIDT2 << 13) | (byIDT3 << 5) | (byIDT4 >> 3);
    }
    else
    {
        dwID = (byIDT1 << 3) | (byIDT2 >> 5);
    }
    return dwID;
}

/**************************************************************************************************
    Description: Create CRC by XOR operation between control byte, message length
                 and all data bytes
---------------------------------------------------------------------------------------------------
    Inputs: pbyMessage - whole message array
            dwLength - Count of bytes in message
    Output: CRC code
**************************************************************************************************/
BYTE CDlgMain::CRC(BYTE *pbyMessage, DWORD dwLength)
{
    BYTE byCRC = 0;
    byCRC ^= pbyMessage[cPosCONTROL];
    byCRC ^= pbyMessage[cPosLENGTH];
    for (DWORD dwIndex = 0; dwIndex < dwLength; dwIndex++)
    {
        byCRC ^= pbyMessage[cPosData + dwIndex];
    }
    return byCRC;
}

/**************************************************************************************************
    Description: Display arrival time, tag and data from message array and increase counters
---------------------------------------------------------------------------------------------------
    Inputs: pbyMessage - complete message
            dwLength - Message length
    Output: -
**************************************************************************************************/
VOID CDlgMain::DisplayMessage(BYTE *pbyMessage, DWORD dwLength)
{
    if ((pbyMessage[cPosSTART1] == cSTART1) &&
        (pbyMessage[cPosSTART2] == cSTART2) &&
        (pbyMessage[cPosEND1 + pbyMessage[cPosLENGTH]] == cEND1) &&
        (pbyMessage[cPosEND2 + pbyMessage[cPosLENGTH]] == cEND2) &&
        (CRC(pbyMessage, dwLength) == pbyMessage[cPosCRC + pbyMessage[cPosLENGTH]]))
    {
        CString strTmp;
        INT nTmp;
        switch (pbyMessage[cPosCONTROL])
        {
        case cCAN_RX:
        case cCAN_TX:
            //Display arrival time
            m_ControlEdit_ArrivalTime.SetWindowText(TimeToStr(pbyMessage[5],
                                                              pbyMessage[6],
                                                              pbyMessage[7],
                                                              256 * pbyMessage[8] + pbyMessage[9]));

            //Display ID
            BOOL bIDE, bRemote; //Not used
            m_ControlEdit_IdReceive.SetWindowText(IntToStrHex(DecodeID(pbyMessage[10],
                                                                       pbyMessage[11],
                                                                       pbyMessage[12],
                                                                       pbyMessage[13],
                                                                       pbyMessage[14],
                                                                       &bIDE, &bRemote)));

            //Display data
            m_ControlEdit_DataReceive.SetWindowText(DataToStr(&pbyMessage[15], pbyMessage[14]));

            //Increment message counter
            m_ControlLabel_Count.GetWindowText(strTmp.GetBuffer(MAX_COUNT_ERROR_LENGTH), MAX_COUNT_ERROR_LENGTH);
            nTmp = StrToInt(strTmp) + 1;
            m_ControlLabel_Count.SetWindowText(IntToStr(nTmp));
            break;
        case cCAN_ERROR:
            //Increment error counter
            m_ControlLabel_Errors.GetWindowText(strTmp.GetBuffer(MAX_COUNT_ERROR_LENGTH), MAX_COUNT_ERROR_LENGTH);
            nTmp = StrToInt(strTmp) + 1;
            m_ControlLabel_Errors.SetWindowText(IntToStr(nTmp));
            break;
        }
    }
    else
    {
        MessageBox(ResStr(IDS_E_MESSAGE_CORRUPTED), ResStr(IDS_ERROR), MB_OK | MB_ICONERROR);
    }
}

/**************************************************************************************************
    Description: Decode and display received data
---------------------------------------------------------------------------------------------------
    Inputs: pbyData - received data array
            dwCount - count of received bytes
    Output: -
**************************************************************************************************/
VOID CDlgMain::OnDataReceived(BYTE *pbyData, DWORD dwCount)
{
    if (m_dwRxDataBufferPos + dwCount > RX_BUFFER_SIZE)
    {
        MessageBox(ResStr(IDS_E_BUFFER_TOO_SMALL), ResStr(IDS_ERROR), MB_OK | MB_ICONERROR);
    }

    //Add data to receive buffer
    for (DWORD dwIndex = 0; dwIndex < dwCount; dwIndex++)
    {
        m_byRxDataBuffer[m_dwRxDataBufferPos++] = pbyData[dwIndex];
    }

    while (m_dwRxDataBufferPos > 0)
    {
        //Finding messages (start1, star2, direction, length, data, end1, end2)
        DWORD dwWrongBytes = 0;
        BOOL bMsgFound = FALSE;
        while (dwWrongBytes <= m_dwRxDataBufferPos)
        {
            if ((m_byRxDataBuffer[dwWrongBytes + cPosSTART1] == cSTART1) &&
                (m_byRxDataBuffer[dwWrongBytes + cPosSTART2] == cSTART2) &&
                (m_byRxDataBuffer[dwWrongBytes + cPosEND1 + m_byRxDataBuffer[dwWrongBytes + cPosLENGTH]] == cEND1) &&
                (m_byRxDataBuffer[dwWrongBytes + cPosEND2 + m_byRxDataBuffer[dwWrongBytes + cPosLENGTH]] == cEND2))
            {
                //Check is message is complete in buffer
                if ((dwWrongBytes + cMsgLENGTH + m_byRxDataBuffer[dwWrongBytes + cPosLENGTH]) <= m_dwRxDataBufferPos)
                {
                    bMsgFound = TRUE;
                }
                break;
            }
            else
            {
                dwWrongBytes++;
            }
        }

        //Check if buffer full and no message was found
        if (dwWrongBytes == RX_BUFFER_SIZE)
        {
            //Delete RX buffer
            m_dwRxDataBufferPos = 0;
            //Display error
            MessageBox(ResStr(IDS_E_BUFFER_FULL_OF_RUBBISH), ResStr(IDS_ERROR), MB_OK | MB_ICONERROR);
        }

        if (bMsgFound)
        {
            //Remove data that are not correct message
            if (dwWrongBytes > 0)
            {
                for (DWORD dwIndex = 0; dwIndex < m_dwRxDataBufferPos; dwIndex++)
                {
                    m_byRxDataBuffer[dwIndex] = m_byRxDataBuffer[dwIndex + dwWrongBytes];
                }
                m_dwRxDataBufferPos -= cMsgLENGTH + dwWrongBytes;
            }
            //Display message
            DWORD dwMsgLength = m_byRxDataBuffer[cPosLENGTH];
            DisplayMessage(m_byRxDataBuffer, dwMsgLength);
            //Remove this message from buffer
            for (DWORD dwIndex = 0; dwIndex < m_dwRxDataBufferPos; dwIndex++)
            {
                m_byRxDataBuffer[dwIndex] = m_byRxDataBuffer[dwIndex + cMsgLENGTH + dwMsgLength];
            }
            m_dwRxDataBufferPos -= cMsgLENGTH + dwMsgLength;
        }
        else
        {
            break;
        }
    }
}

/**************************************************************************************************
    Description: Periodically checking for device presence
---------------------------------------------------------------------------------------------------
    Inputs: pointer to main dialog class
    Output: Thread status - not used
**************************************************************************************************/
#pragma warning (disable: 4312) //non pointer value to poiter value, not safe but it works
DWORD WINAPI ThreadTimerRutine(LPVOID lpParameter)
{
    CDlgMain *pDlgMain = (CDlgMain *)lpParameter;
    FT_STATUS ftStatus = FT_OK;

    Sleep(500);
    while (!pDlgMain->m_bThreadTimerTerminate)
    {
        if (pDlgMain->m_bDevicePresent)
        {
            DWORD dwPendingBytesCount;
            ftStatus = FT_GetQueueStatus(pDlgMain->m_FT_Handle, &dwPendingBytesCount);
            if (!FT_SUCCESS(ftStatus))
            {
                pDlgMain->m_bDevicePresent = FALSE;
                pDlgMain->SetWindowText(ResStr(IDS_TITLE));
            }
        }
        else
        {
            DWORD dwDeviceCount;
            if (pDlgMain->m_FT_Handle != NULL)
            {
                ftStatus = FT_Close(pDlgMain->m_FT_Handle);
            }
            if (FT_SUCCESS(ftStatus))
            {
                ftStatus = FT_ListDevices(&dwDeviceCount, NULL, FT_LIST_NUMBER_ONLY);
                if (FT_SUCCESS(ftStatus))
                {
                    for (DWORD nIndex = 0; nIndex < dwDeviceCount; nIndex++)
                    {
                        BYTE byDeviceStringBuffer[50];
                        ftStatus = FT_ListDevices((PVOID)nIndex, &byDeviceStringBuffer, FT_OPEN_BY_SERIAL_NUMBER | FT_LIST_BY_INDEX);
                        if (FT_SUCCESS(ftStatus))
                        {
                            FT_STATUS ftStatus = FT_OpenEx(&byDeviceStringBuffer, FT_OPEN_BY_SERIAL_NUMBER, &(pDlgMain->m_FT_Handle));
                            if (FT_SUCCESS(ftStatus))
                            {
                                ftStatus = FT_ResetDevice(pDlgMain->m_FT_Handle); //warning - this will destroy all pending data
                                if (FT_SUCCESS(ftStatus))
                                {
                                    ftStatus = FT_SetTimeouts(pDlgMain->m_FT_Handle, 10, 10); //read and write timeouts = 10ms (1..500)ms
                                    if (FT_SUCCESS(ftStatus))
                                    {
                                        pDlgMain->m_bDevicePresent = TRUE;
                                        pDlgMain->SetWindowText(ResStr(IDS_TITLE) + TEXT(" - Device Present"));
                                        break; //Successfully opened
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }

        do
        {
            Sleep(100);
        }
        while (pDlgMain->m_bThreadTimerPaused);
    }
        
    return 0;
}
#pragma warning (default: 4312)

/**************************************************************************************************
    Description: Periodically checking for device presence
---------------------------------------------------------------------------------------------------
    Inputs: pointer to main dialog class
    Output: Thread status - not used
**************************************************************************************************/
DWORD WINAPI ThreadReceiveRutine(LPVOID lpParameter)
{
    CDlgMain *pDlgMain = (CDlgMain *)lpParameter;
    FT_STATUS ftStatus = FT_OK;
    HANDLE hRxEvent = CreateEvent(NULL, FALSE, FALSE, TEXT(""));

    while (!pDlgMain->m_bThreadReceiveTerminate && !pDlgMain->m_bDevicePresent)
    {
        Sleep(100);
    }
    if (pDlgMain->m_bDevicePresent)
    {
        ftStatus = FT_SetEventNotification(pDlgMain->m_FT_Handle, FT_EVENT_RXCHAR, hRxEvent);
        if (!FT_SUCCESS(ftStatus))
        {
            pDlgMain->MessageBox(ResStr(IDS_E_CANT_INIT_RX_EVENT), ResStr(IDS_ERROR), MB_OK | MB_ICONERROR);
            pDlgMain->SendMessage(WM_CLOSE); //Close application
        }
    }

    while (!pDlgMain->m_bThreadReceiveTerminate)
    {
        if (pDlgMain->m_bDevicePresent)
        {
            WaitForSingleObject(hRxEvent, 250);

            BOOL bTmp = pDlgMain->m_bThreadReceivePaused;
            pDlgMain->m_bThreadReceivePaused = TRUE;
            DWORD dwRxBytes, dwTxBytes, dwEventDWord;
            ftStatus = FT_GetStatus(pDlgMain->m_FT_Handle, &dwRxBytes, &dwTxBytes, &dwEventDWord);

            BYTE byBuffer[500];
            DWORD dwReceivedBytes = 0;
            //while (FT_SUCCESS(ftStatus) && (dwTxBytes & FT_EVENT_RXCHAR) != 0 && dwRxBytes > 0)
            if (FT_SUCCESS(ftStatus) && (dwEventDWord & FT_EVENT_RXCHAR) != 0 && dwRxBytes > 0)
            {
                ftStatus = FT_Read(pDlgMain->m_FT_Handle, byBuffer, 500, &dwReceivedBytes);
                if (FT_SUCCESS(ftStatus))
                {
                    pDlgMain->OnDataReceived(byBuffer, dwReceivedBytes);
                }
            }
            pDlgMain->m_bThreadReceivePaused = bTmp;
        }
        else
        {
            Sleep(111);
        }
    }

    return 0;
}
